; IPC5 Domain: Pathways ComplexPreferences
; Authors: Yannis Dimopoulos, Alfonso Gerevini and Alessandro Saetti 
; modifications to this version by Amanda Coles and Andrew Coles see http://www.personal.cis.strath.ac.uk/~amanda/Preferences for details.

(define (domain Pathways-ComplexPreferences) 
(:requirements :typing :fluents :preferences :constraints)  

(:types level molecule - object
	simple complex - molecule) 

(:predicates (association-reaction ?x1 ?x2 - molecule ?x3 - complex)
             (catalyzed-association-reaction ?x1 ?x2 - molecule ?x3 - complex)
             (catalyzed-self-association-reaction ?x1 - molecule ?x3 - complex) ;; Fact added to fix self-association bug
             (synthesis-reaction ?x1 ?x2 - molecule)
             (possible ?x - molecule)
             (chosen ?x - simple))

(:functions (available ?x - molecule)
            (need-for-association ?x1 ?x2 - molecule ?x3 - complex)
            (need-for-catalyzed-association ?x1 ?x2 - molecule ?x3 - complex)
            (need-for-catalyzed-self-association ?x1 - molecule ?x3 - complex) ;; Function added to fix self-association bug
            (returned-after-catalyzed-self-association ?x1 - molecule ?x3 - complex)  ;; Function added to fix self-association bug
            (need-for-synthesis ?x1 ?x2 - molecule)
            (prod-by-association ?x1 ?x2 - molecule ?x3 - complex)
            (prod-by-catalyzed-association ?x1 ?x2 - molecule ?x3 - complex)
            (prod-by-catalyzed-self-association ?x1 - molecule ?x3 - complex)  ;; Function added to fix self-association bug
            (prod-by-synthesis ?x1 ?x2 - molecule)
            (num-subs))

(:action choose
 :parameters (?x - simple)
 :precondition (and (possible ?x))
 :effect (and (not (possible ?x)) (chosen ?x) )
)

(:action initialize
 :parameters (?x - simple)
 :precondition (and (chosen ?x))
 :effect (and (increase (available ?x) 1))
)

(:action associate
 :parameters (?x1 ?x2 - molecule ?x3 - complex)
 :precondition (and (>= (available ?x1) (need-for-association ?x1 ?x2 ?x3))
		 (>= (available ?x2) (need-for-association ?x2 ?x1 ?x3))
		 (association-reaction ?x1  ?x2  ?x3))
 :effect (and (decrease (available ?x1) (need-for-association ?x1 ?x2 ?x3)) 
	      (decrease (available ?x2) (need-for-association ?x2 ?x1 ?x3))
	      (increase (available ?x3) (prod-by-association ?x1 ?x2 ?x3)))
)

(:action associate-with-catalyze
 :parameters (?x1 ?x2 - molecule ?x3 - complex)
 :precondition (and (>= (available ?x1) (need-for-catalyzed-association ?x1 ?x2 ?x3)) 
                    (>= (available ?x2) (need-for-catalyzed-association ?x2 ?x1 ?x3))
                    (catalyzed-association-reaction ?x1 ?x2 ?x3))
 :effect (and (decrease (available ?x1) (need-for-catalyzed-association ?x1 ?x2 ?x3))
              (increase (available ?x3) (prod-by-catalyzed-association ?x1 ?x2 ?x3)))
)

;; Action added to fix self-association bug
(:action self-associate-with-catalyze
 :parameters (?x1 - molecule ?x3 - complex)
 :precondition (and (>= (available ?x1) (need-for-catalyzed-self-association ?x1 ?x3))
                    (catalyzed-self-association-reaction ?x1 ?x3))
 :effect (and (decrease (available ?x1) (- (need-for-catalyzed-self-association ?x1 ?x3)
                                           (returned-after-catalyzed-self-association ?x1 ?x3)))
              (increase (available ?x3) (prod-by-catalyzed-self-association ?x1 ?x3)))
)


(:action synthesize
 :parameters (?x1 ?x2 - molecule)
 :precondition (and (>= (available ?x1) (need-for-synthesis ?x1 ?x2))
                    (synthesis-reaction ?x1 ?x2))
 :effect (and (increase (available ?x2) (prod-by-synthesis ?x1 ?x2)))
)

)

